#version 430 core

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 3) in vec2 uv;

uniform mat4 M;
uniform mat4 V;
uniform mat4 P;
uniform bool depthPass;
uniform mat4 VP_light;
uniform mat3 NormalM;
uniform bool clipPlaneEnabled;
uniform vec4 clipPlane;
uniform vec3 aabb_minV;
uniform vec3 aabb_maxV;
uniform float animT;

out vec3 eyeDirection_cameraspace;
out vec3 normal_cameraspace;
out vec2 fragUV;
out vec3 fragPosition_cameraspace;
out vec3 fragPosition_worldspace;

void main() {
	vec4 pos_worldspace = M * vec4(position, 1);

	// animate with a shear function
	float m_unit = 0.02;
	vec3 shearDirection = vec3(0, 0, 1) * m_unit;
	float t = (position.y - aabb_minV.y) / (abs(aabb_maxV.y - aabb_minV.y));
	t = clamp(t, 0.0, 1.0);

	pos_worldspace = pos_worldspace + vec4(shearDirection * pow(t + 0.3, 2.0) * animT, 0);

	// clip if enabled (refraction)
	if(clipPlaneEnabled) {
		gl_ClipDistance[0] = dot(pos_worldspace, clipPlane + vec4(0, 0, 0, 0));
	}

	// calculate vertex position in world and camera space
	fragPosition_worldspace = pos_worldspace.xyz;
	fragPosition_cameraspace = (V * pos_worldspace).xyz;
	eyeDirection_cameraspace = vec3(0) - fragPosition_cameraspace;	
	
	// calculate vertex normal in camera space
	normal_cameraspace = ( V * vec4(NormalM * normal, 0) ).xyz;
	
	// pass uv coords 
	fragUV = uv;

	// set ndc coords
	if(depthPass) {
		gl_Position =  VP_light * pos_worldspace;
	} else {	
		gl_Position = P * V * pos_worldspace;
	}
}