#version 430 core

in vec2 fragUV;

layout(binding=0) uniform sampler2D sceneTex;
layout(binding=1) uniform sampler2D brightPassTex;


layout(binding=2) uniform sampler2D lenseFlareTex;
layout(binding=3) uniform sampler2D lenseDirtTex;
layout(binding=4) uniform sampler2D lenseStarTex;

uniform mat3 starMat;
uniform float tFade;
uniform float fadeDuration;

uniform bool nightMode;

out vec4 color;

void main() {
	/////////////////
	// BLOOM
	/////////////////

	vec3 scene_t = texture(sceneTex, fragUV).rgb;
	vec3 bp_t = texture(brightPassTex, fragUV).rgb;
	vec3 color_t = scene_t + bp_t;

	color = vec4(color_t, 1);
	
	/////////////////
	// LENSE FLARE
	/////////////////

	if(!nightMode) {
		vec4 lensModulator = texture(lenseDirtTex, fragUV);

	    vec2 lensStarUV = (starMat * vec3(fragUV, 1.0)).xy;
	    lensModulator += texture(lenseStarTex, lensStarUV);

	    vec4 lenseflare = texture(lenseFlareTex, fragUV) * lensModulator * 10;
	    color += lenseflare;
	}

	/////////////////
	// FILMIC TONE MAPPING APPROXIMATION + GAMMA CORRECTION
	/////////////////

	vec3 c_max = max(color.rgb - 0.004, 0);
	color = vec4((c_max * (6.2 * c_max + 0.2)) / (c_max * (6.2 * c_max + 1.7) + 0.09), 1);


	/////////////////
	// FADE OUT / FADE IN
	/////////////////	

	vec3 fadeOutColor = vec3(0);
	float t_interp = tFade / fadeDuration;
	color.rgb = color.rgb * (1-t_interp) + fadeOutColor * t_interp;
}