#version 430 core

#define MAX_LIGHTS 10

in vec3 eyeDirection_cameraspace;
in vec3 normal_cameraspace;
in vec3 fragPosition_cameraspace;

uniform struct PointLight {
	vec3 position;
	vec3 diffColor;
	vec3 specColor;
	vec3 attenuation;
} pointLights[MAX_LIGHTS];

uniform struct DirectionalLight {
	vec3 direction;
	vec3 diffColor;
	vec3 specColor;
} directionalLights[MAX_LIGHTS];

uniform uint nbPointLights;
uniform uint nbDirectionalLights;

uniform vec3 matAmbColor;
uniform vec3 matDiffColor;
uniform vec3 matSpecColor;
uniform float matShininess;
uniform vec3 matIlluminationWeights;

uniform mat4 V;

out vec4 color;

void main() {	
	vec3 n = normalize(normal_cameraspace); //normalize because interpolated

	vec3 diffuseIntensities = vec3(0);
	vec3 specularIntensities = vec3(0);
	for (uint i = 0; i < nbDirectionalLights; i++) {		
		vec3 l = normalize(vec3(V * vec4(directionalLights[i].direction, 0))); //normalize because interpolated
		// cosine of the angle between the normal and the light direction (lambert)
		float cosTheta = clamp(dot(l,n), 0, 1);

		diffuseIntensities += directionalLights[i].diffColor * cosTheta;

		// calculate specular parts

		// Eye vector (towards the camera)
		vec3 E = normalize(eyeDirection_cameraspace);
		// Direction in which the triangle reflects the light
		vec3 R = reflect(-l,n);
		// Cosine of the angle between the Eye vector and the Reflect vector
		float cosAlpha = clamp( dot( E,R ), 0, 1 );

		specularIntensities += directionalLights[i].specColor * pow(cosAlpha, matShininess);
	}


	for (uint i = 0; i < nbPointLights; i++) {			
		vec3 l = (V * vec4(pointLights[i].position, 1)).xyz - fragPosition_cameraspace; 
		float d = length(l);
		l = normalize(l);
		// cosine of the angle between the normal and the light direction (lambert)
		float cosTheta = clamp(dot(l,n), 0, 1);
		
		float totalAttenuation = pointLights[i].attenuation[0] + pointLights[i].attenuation[1] * d + pointLights[i].attenuation[2] * d * d;

		diffuseIntensities += (1.0 / totalAttenuation) * pointLights[i].diffColor * cosTheta;

		// calculate specular parts

		// Eye vector (towards the camera)
		vec3 E = normalize(eyeDirection_cameraspace);
		// Direction in which the triangle reflects the light
		vec3 R = reflect(-l,n);
		// Cosine of the angle between the Eye vector and the Reflect vector,
		float cosAlpha = clamp( dot( E,R ), 0,1 );

		specularIntensities += (1.0 / totalAttenuation) * pointLights[i].specColor * pow(cosAlpha, matShininess);
	}
	
	color = vec4(
				matAmbColor * matIlluminationWeights.x + 
				(matDiffColor * diffuseIntensities) * matIlluminationWeights.y + 
				(matSpecColor * specularIntensities) * matIlluminationWeights.z, 
			1);
}