#version 430 core

noperspective in vec2 fragUV;

layout(binding=0) uniform sampler2D sceneTex;
layout(binding=1) uniform sampler2D lenseColorTex;

uniform int ghostsNumber;
uniform float ghostDispersalFactor;
uniform float haloWidth;
uniform float distortation;

out vec4 color;

 vec4 textureDistorted(
      in sampler2D tex,
      in vec2 texcoord,
      in vec2 direction, // direction of distortion
      in vec3 distortion // per-channel distortion factor
   ) {
      return vec4(
         texture(tex, texcoord + direction * distortion.r, 1).r,
         texture(tex, texcoord + direction * distortion.g, 1).g,
         texture(tex, texcoord + direction * distortion.b, 1).b,
         1
      );
   }

void main() {
	// flip tex coordinates
	vec2 uv = -fragUV + vec2(1.0);
	vec2 texelSize = 1.0 / vec2(textureSize(sceneTex, 0));
	vec3 distortion = vec3(-texelSize.x * distortation, 0.0, texelSize.x * distortation);

	// ghost vector to image centre:
    vec2 ghostVec = (vec2(0.5) - uv) * ghostDispersalFactor;
	
	// create ghosts
    vec4 result = vec4(0.0);
    for (int i = 0; i < ghostsNumber; ++i) { 
		// fract to wrap around image
        vec2 offset = fract(uv + ghostVec * float(i));

		float weight = length(vec2(0.5) - offset) / length(vec2(0.5));
		weight = pow(1.0 - weight, 10.0);
  
        result += textureDistorted(sceneTex, offset, normalize(ghostVec), distortion) * weight;
    }
	
	result *= texture(lenseColorTex, vec2(length(vec2(0.5) - uv) / length(vec2(0.5)), 0.5) );
 
	// sample halo:
   vec2 haloVec = normalize(ghostVec) * haloWidth;
   float weight = length(vec2(0.5) - fract(uv + haloVec)) / length(vec2(0.5));
   weight = pow(1.0 - weight, 5.0);
   result += textureDistorted(sceneTex, fract(uv + haloVec), normalize(ghostVec), distortion) * weight;
	
    color = result;
	//color = texture(sceneTex, fragUV, 1);
}