#version 430 core

layout(location = 0) in vec4 position;

uniform mat4 MV;
uniform mat4 P;

uniform float particleSize;


out vec2 UV;
out vec4 color;

out vec3 eyeDirection_cameraspace;
out vec3 fragPosition_cameraspace;
out vec3 position_modelspace;

void main() {
	// generate UVs to be indexed with the last 2 bits (= index mod 4) of the vertex index in the vertex shader
	vec2 UVs[] = vec2[4](
		vec2(0, 0),
		vec2(1, 0),
		vec2(1, 1),
		vec2(0, 1));

	// use gl_VertexID (= id from index buffer) to get particle position of SSBO and for index of UV
	vec4 particlePos = vec4(vec3(position), 1); //positions[gl_InstanceID];

	UV = UVs[gl_VertexID];

	vec4 particlePosEye = MV * particlePos;
	vec4 vertexPosEye = particlePosEye + vec4((UV * 2.0f - 1.0f) * clamp(particleSize * position.y/40, particleSize, particleSize*3), 0, 0);

	gl_Position = P * vertexPosEye;

	float particleGreyShade = 0.08;
	color = vec4(vec3(particleGreyShade), 1);

	
	// calculate vertex position in camera space
	vec3 vertexPosition_cameraspace = (vertexPosEye).xyz;
	eyeDirection_cameraspace = vec3(0,0,0) - vertexPosition_cameraspace;
	fragPosition_cameraspace = vertexPosition_cameraspace;
	position_modelspace = particlePos.xyz;
}