#version 430 core

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 3) in vec2 uv;
layout(location = 5) in vec3 tangent;
layout(location = 6) in vec3 bitangent;

uniform mat4 MVP;
uniform mat4 M;
uniform mat4 V;
uniform bool depthPass;
uniform mat4 VP_light;
uniform mat3 NormalM;
uniform bool clipPlaneEnabled;
uniform vec4 clipPlane;

out vec3 eyeDirection_cameraspace;
out vec3 normal_cameraspace;
out vec3 tangent_cameraspace;
out vec3 bitangent_cameraspace;
out vec2 fragUV;
out vec3 fragPosition_cameraspace;
out vec3 fragPosition_worldspace;

void main() {
	if(depthPass) {
		gl_Position =  VP_light * M * vec4(position, 1);
	} else {	
		gl_Position = MVP * vec4(position, 1);

		vec4 pos_worldspace = M * vec4(position, 1);
		if(clipPlaneEnabled) {
			gl_ClipDistance[0] = dot(pos_worldspace, clipPlane + vec4(0, 0, 0, 0));
		}

		// calculate vertex position in world and camera space
		fragPosition_worldspace = pos_worldspace.xyz;
		fragPosition_cameraspace = (V * pos_worldspace).xyz;
		eyeDirection_cameraspace = vec3(0) - fragPosition_cameraspace;
		tangent_cameraspace = (V * M * vec4(tangent, 0)).xyz;
		bitangent_cameraspace = (V * M * vec4(bitangent, 0)).xyz;
		
		// calculate vertex normal in camera space
		normal_cameraspace = ( V * vec4(NormalM * normal, 0) ).xyz;
		
		// pass uv coords 
		fragUV = uv;
	}
}