#version 430 core

// this shader produces 4 vertices as outputs
layout(vertices = 4) out;

uniform vec2 screen_size;
uniform mat4 MVP;
uniform mat4 M;
uniform mat4 V;
uniform float lodFactor;
uniform float lodDistance;

// convert device vec4 in device normal space
vec4 project(vec4 vertex) {
	vec4 result = MVP * vertex;
	result /= result.w;
	return result;
}

vec3 view(vec4 vertex) {
	vec4 result = V * M * vertex;
	return result.xyz;
}

// test vertex in device normal space against view frustum (don't calculate vertices outside the view frustum)
bool onscreen(vec4 vertex) {
	if (vertex.z > -1.0) {
		return true;
	}
	bvec2 lt = greaterThan(vertex.xy, vec2(-1.0));
	bvec2 gt = lessThan(vertex.xy, vec2(1.0));
	return lt.x || lt.y || gt.x || gt.y;
	//return any(
    //    lessThan(vertex.xy, vec2(-1.7)) ||
    //    greaterThan(vertex.xy, vec2(1.7))
    //); ));
	//return lt.x && lt.y && gt.x && gt.y;
}

void main() {
	//gl_InvocationID identifies which vertex is being processed
	#define id gl_InvocationID
	gl_out[id].gl_Position = gl_in[id].gl_Position;
	if (id == 0) {
		vec4 v0 = project(gl_in[0].gl_Position);
		vec4 v1 = project(gl_in[1].gl_Position);
		vec4 v2 = project(gl_in[2].gl_Position);
		vec4 v3 = project(gl_in[3].gl_Position);

		// divide the patch zero times if out of view frustum
		// look if any vertx is in the view frustum, than divide it
		if (any(bvec4(
			onscreen(v0),
			onscreen(v1),
			onscreen(v2),
			onscreen(v3)
			))
			// && v0.z > 0.9999 test --> z values lie between 0.99 and 1.0, the more far the scene is away, the greater becomes z --> we look into the positive z axis?
			) {

			vec3 v0_v = view(gl_in[0].gl_Position);
			vec3 v1_v = view(gl_in[1].gl_Position);
			vec3 v2_v = view(gl_in[2].gl_Position);
			vec3 v3_v = view(gl_in[3].gl_Position);

			float mult = 1.0;

			float e0 = (1.0 - clamp(mult * length(v1_v) + mult * length(v2_v), 1.0 , lodDistance) / lodDistance) * lodFactor; // bottom side
			float e1 = (1.0 - clamp(mult * length(v0_v) + mult * length(v1_v), 1.0 , lodDistance) / lodDistance) * lodFactor; // right side 
			float e2 = (1.0 - clamp(mult * length(v3_v) + mult * length(v0_v), 1.0 , lodDistance) / lodDistance) * lodFactor; // top side
			float e3 = (1.0 - clamp(mult * length(v2_v) + mult * length(v3_v), 1.0 , lodDistance) / lodDistance) * lodFactor; // left side

			e0 = clamp(e0,1,63);
			e1 = clamp(e1,1,63);
			e2 = clamp(e2,1,63);
			e3 = clamp(e3,1,63);

			//gl_TessLevelInner defines how often the inside of the patch is divided
			gl_TessLevelInner[0] = mix(e1, e2, 0.0);
			gl_TessLevelInner[1] = mix(e0, e3, 0.0);

			//gl_TessLevelOuter defines how often the edge is divided
			//countermeasure for artifact multiply edge tessellation
			gl_TessLevelOuter[0] = e0;
			gl_TessLevelOuter[1] = e1;
			gl_TessLevelOuter[2] = e2;
			gl_TessLevelOuter[3] = e3;
		}
		else {
			//gl_TessLevelInner defines how often the inside of the patch is divided
			gl_TessLevelInner[0] = 0;
			gl_TessLevelInner[1] = 0;

			//gl_TessLevelOuter defines how often the edge is divided
			gl_TessLevelOuter[0] = 0;
			gl_TessLevelOuter[1] = 0;
			gl_TessLevelOuter[2] = 0;
			gl_TessLevelOuter[3] = 0;
		}
	}
}