#version 430 core

layout(quads, fractional_odd_spacing, ccw) in;



uniform mat3 NormalM;
uniform mat4 M;
uniform mat4 V;
uniform mat4 MVP;

uniform sampler2D heightMap;
uniform sampler2D normalMap;

out vec3 eyeDirection_cameraspace;
out vec3 normal_cameraspace;
out vec3 fragPosition_cameraspace;

out vec2 UV;

float calcHeight(vec2 position) {
	vec4 textureHeight = texture(heightMap,  position);
	float height = textureHeight.r;
	return height  / 2.0f;
}

void main() {
    float u = gl_TessCoord.x;
    float v = gl_TessCoord.y;

    vec4 a = mix(gl_in[1].gl_Position, gl_in[0].gl_Position, u);
    vec4 b = mix(gl_in[2].gl_Position, gl_in[3].gl_Position, u);
    vec4 position = mix(a, b, v);
    UV = (position.xz / 2.0 + vec2(0.5, 0.5)) ; //move our texture coords, because the meshes lie between -0.5 and 0.5, while the texture coordinates must lie between 0.0 and 1.0
   
    float height = calcHeight(UV);

	position = vec4(position.x, height, position.z, 1.0);
	gl_Position = MVP * position;
	
	vec3 normal = texture(normalMap,  UV).xyz;

	// -- shading --

	// calculate vertex position in camera space
	vec3 vertexPosition_cameraspace = ( V * M * position).xyz;
	eyeDirection_cameraspace = vec3(0,0,0) - vertexPosition_cameraspace;
	fragPosition_cameraspace = vertexPosition_cameraspace;

	// calculate vertex normal in camera space
	normal_cameraspace = normalize(( V * vec4(NormalM * normal, 0) ).xyz);
}