#version 430 core

layout(quads, fractional_odd_spacing, ccw) in;



uniform mat3 NormalM;
uniform mat4 M;
uniform mat4 V;
uniform mat4 MVP;

uniform float waveOffset;
uniform sampler2D surfaceMap;
uniform sampler2D surfaceMap2;

out vec3 eyeDirection_cameraspace;
out vec3 normal_cameraspace;
out vec3 fragPosition_cameraspace;

out vec2 UV;

out vec3 normal;

float rand(vec2 co){
  return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

float calcWaterHeight(vec2 position) {
	float height1 = texture(surfaceMap, clamp(mod(vec2(position.x + waveOffset, position.y) * 2, 1.0), 0.001, 0.999)).r / 192.0;
	float height2 = texture(surfaceMap, clamp(mod(vec2(position.x, position.y  + waveOffset * 1.5) * 2, 1.0), 0.001, 0.999)).r / 256.0;
	float height3 = texture(surfaceMap, clamp(mod(vec2(position.x + waveOffset, position.y - waveOffset) * 2, 1.0), 0.001, 0.999)).r / 192.0;
	float height4 = texture(surfaceMap, clamp(mod(vec2(position.x - 0.8 * waveOffset, position.y + 1.2 * waveOffset) * 3, 1.0), 0.001, 0.999)).r / 192.0;

	return height1 + height2 + height3 + height4;
}

float calcWaterHeightNormal(vec2 position, vec2 nearPos) {
	float height1 = texture(surfaceMap, clamp(mod(vec2(position.x + waveOffset, position.y) * 2, 1.0) + nearPos, 0.001, 0.999)).r / 192.0;
	float height2 = texture(surfaceMap, clamp(mod(vec2(position.x, position.y  + waveOffset * 1.5) * 2, 1.0) + nearPos, 0.001, 0.999)).r / 256.0;
	float height3 = texture(surfaceMap, clamp(mod(vec2(position.x + waveOffset, position.y - waveOffset) * 2, 1.0) + nearPos, 0.001, 0.999)).r / 192.0;
	float height4 = texture(surfaceMap, clamp(mod(vec2(position.x - 0.8 * waveOffset, position.y + 1.2 * waveOffset) * 3, 1.0) + nearPos, 0.001, 0.999)).r / 192.0;

	float height5 = 0; texture(surfaceMap2, clamp(mod(vec2(position.x + 0.7 * waveOffset, position.y + 0.3 * waveOffset) * 16, 1.0) + nearPos, 0.001, 0.999)).r / 600.0;


	return height1 + height2 + height3 + height4 + height5;
}

void main() {
    float u = gl_TessCoord.x;
    float v = gl_TessCoord.y;

    vec4 a = mix(gl_in[1].gl_Position, gl_in[0].gl_Position, u);
    vec4 b = mix(gl_in[2].gl_Position, gl_in[3].gl_Position, u);
    vec4 position = mix(a, b, v);
    UV = (position.xz / 2.0 + vec2(0.5, 0.5)) ; //move our texture coords, because the meshes lie between -0.5 and 0.5, while the texture coordinates must lie between 0.0 and 1.0
   
    float height = calcWaterHeight(UV);

	float nearPos = 0.005;
	//caluculate normal
	//maybe tranform posUp and posRight
	float heightUp = calcWaterHeightNormal(UV, vec2(nearPos, 0));
	vec4 posUp =  vec4(position.x + nearPos, heightUp, position.z, 1.0);

	float heightRight = calcWaterHeightNormal(UV, vec2(0, nearPos));
	vec4 posRight = vec4(position.x, heightRight, position.z+nearPos, 1.0);

	position = vec4(position.x, height, position.z, 1.0);
	gl_Position = MVP * position;
	
	position = vec4(position.x, calcWaterHeightNormal(UV, vec2(0, 0)), position.z, 1.0);
	
	normal = normalize( cross(vec3(posRight-position), vec3(posUp-position)));

	// -- shading --

	// calculate vertex position in camera space
	vec3 vertexPosition_cameraspace = ( V * M * position).xyz;
	eyeDirection_cameraspace = vec3(0,0,0) - vertexPosition_cameraspace;
	fragPosition_cameraspace = vertexPosition_cameraspace;

	// calculate vertex normal in camera space
	normal_cameraspace = normalize(( V * vec4(NormalM * normal, 0) ).xyz); // - normalize((V * M * position).xyz);
}